// Connection.cpp: implementation of the CConnection class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "MUUDPIC.h"
#include "Connection.h"
#include "MUUDPICDlg.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CConnection::CConnection()
{
  m_connected = false;
}

CConnection::~CConnection()
{

}

bool CConnection::Connect(sockaddr_in loc, const CString& name)
{
  ASSERT(m_dlg); //DIALOG MUST BE SET!!!

  loc.sin_family = AF_INET;
  loc.sin_port = htons(CPORT);

  CSocket conn;
  if (!conn.Create(0, SOCK_STREAM, 0)) {
    m_dlg->DisplayLine("Error in socket creation");
    return true;
  }

  CString msg("Connecting to server at: ");
  msg += inet_ntoa(loc.sin_addr);
  m_dlg->DisplayLine(msg);

  if (!conn.Connect((SOCKADDR*)&loc, sizeof(loc))) {
    AfxMessageBox("Error in socket connect -- Possibly the server is down or not at this address.");
    return true;
  }

  m_dlg->DisplayLine("Connected.  Checking Version");
  msg.Format("%c%s", VERSION, name);
  int sent = conn.Send(msg, msg.GetLength() + 1);
  if (sent == SOCKET_ERROR)
    return true;
  gout += sent;
  m_dlg->UpdateStats();

  m_dlg->DisplayLine("Checking version");
  uchar ver = 0;
  gin += conn.Receive(&ver, sizeof(uchar));

  if (ver == GOODVER) {
    msg.Format("Version %i verified.", int(GOODVER));
  } else {
    msg.Format("Version %i is not same as server's version %i.  Terminating...", int(GOODVER), int(ver));
    conn.Close();
    return true;
  }

  m_dlg->UpdateStats();
  m_dlg->ClearClientList();

  m_dlg->DisplayLine("Receiving Room Information...");
  int NumClis = 0;
  gin += conn.Receive(&NumClis, sizeof(int));
  for (int c=0; c<NumClis; c++) {
    CClient* NewUser = new CClient;
    //Get name
    int NameLen = 0;
    gin += conn.Receive(&NameLen, sizeof(int));
    char* name = new char[NameLen];
    gin += conn.Receive(name, NameLen);
    NewUser->name = name;
    //Get pingtime
    gin += conn.Receive(&NewUser->pingtime, sizeof(NewUser->pingtime));
    m_peers._MoveData(NewUser);

    m_dlg->UpdateStats();
  }
  m_dlg->UpdateClients(&m_peers);

  m_dlg->DisplayLine("Responding to server's ping");
  //Respond to server's ping
  unsigned char temp;
  gin += conn.Receive(&temp, sizeof(temp)); //Ready to ping?
  gout += conn.Send(&temp, sizeof(temp));   //Yes I am
  gin += conn.Receive(&temp, sizeof(temp)); //Wait for the real thing
  gout += conn.Send(&temp, sizeof(temp));   //I got it
  int pingtime;
  gin += conn.Receive(&pingtime, sizeof(pingtime));
  msg.Format("\r\nServer reports a %i ms ping time", pingtime);
  m_dlg->DisplayLine(msg);
  m_dlg->UpdateStats();

  m_dlg->DisplayLine("Moving to UDP broadcast port");

  if (!m_conn.Create(0, SOCK_DGRAM, FD_READ)) {
    AfxMessageBox("Error in socket creation");
    return true;
  }
  loc.sin_port = htons(BPORT); //switch to UDP port
  if (!m_conn.Connect((SOCKADDR*)&loc, sizeof(loc))) {
    AfxMessageBox("Error in socket connect");
    return true;
  }

  //Send the server our contact info
  m_dlg->DisplayLine("Sending contact information");
  int len = sizeof(loc);
  m_conn.GetSockName((SOCKADDR*)&loc, &len); //get our port
  gout += conn.Send(&loc.sin_port, sizeof(loc.sin_port));
  m_dlg->UpdateStats();

  conn.Close();
  m_connected = true; //we are connected!!!
  return false;
}

void CConnection::Disconnect()
{
  if (m_connected) {
    CString msg;
    msg.Format("%c", HDR_EXIT);
    SendMessage(msg);
    m_conn.Close();
    m_connected = false;
    m_peers.Clear();
    m_dlg->ClearClientList();
  } else
    m_dlg->DisplayLine("Already disconnected!");
}

bool CConnection::SendMessage(const CString& cmsg)
{
  int msglen = cmsg.GetLength() + 1;
  const char* msg = (LPCTSTR)cmsg;
  int sent = 0;                //Amount of data sent so far
  while (sent < msglen) {
    sent += m_conn.Send(&msg[sent], msglen-sent);
    if (sent == SOCKET_ERROR)
      return true;
  }
  gout += sent;                //Update connection stats
  m_dlg->UpdateStats();  
  return false;
}

void CConnection::SetDialog(CMUUDPICDlg *dlg)
{
  m_dlg = dlg;
  m_conn.SetDialog(dlg, this);
}

void CConnection::NewJoin(const CString &msg)
{
  ASSERT(m_dlg);

  CClient* newcli = new CClient();
  int numchars = 0;
  sscanf(msg, " %i %n", &newcli->pingtime, &numchars);
  newcli->name = &((LPCTSTR)msg)[numchars];
  m_peers._MoveData(newcli);

  m_dlg->UpdateClients(&m_peers);
}

void CConnection::LeaveCli(const CString &msg)
{
  //Client's name is in msg
  m_peers.SetToStart();
  int l = m_peers.Length();
  for (int c=0; c<l; c++) {
    if (msg == m_peers._GetNext()->name)
      m_peers._DeleteThis(); //This is okay since only unique names are allowed
    else
      m_peers._Advance();
  }
  m_dlg->UpdateClients(&m_peers);
}

void CConnection::PingResponse()
{
  CString msg;
  msg.Format("%c", HDR_PING);
  SendMessage(msg);
}

void CConnection::ServerDisconnect()
{
  //This happens when the server leaves and sends a SERVEXIT packet
  m_conn.Close();
  m_connected = false;
  m_dlg->DisplayLine("The server was terminated.");
  m_dlg->DisplayLine("Disconnected.");
  m_peers.Clear();
  m_dlg->ClearClientList();
}

void CConnection::Talk(const CString &msg)
{
  //it's possible for the user to type a message w/o being connected
  if (m_connected) {
    CString temp;
    temp.Format("%c%s", HDR_MSG, msg);
    SendMessage(temp);
  } else
    m_dlg->DisplayLine("You are not connected to the server!");
}
